from . import settings
from typing import Optional, List

class Content:
    Empty: Content
    Splitter: Content
    def __init__(self, c: str) -> Content: ...

class Cell:
    def __init__(self, c: Content) -> Cell:
        self.content: Content
        self.overflow: settings.Overflow
        self.width: Optional[int]
        self.align: settings.Align
        self.padding: settings.Padding
        self.merge: Optional[int]
        self.formatter: List[settings.Formatter]
    
    def with_content(self, content: Content) -> Cell: ...
    def with_overflow(self, overflow: settings.Overflow) -> Cell: ...
    def with_width(self, width: Optional[int]) -> Cell: ...
    def with_align(self, align: settings.Align) -> Cell: ...
    def with_padding(self, padding: settings.Padding) -> Cell: ...
    def with_merge(self, merge: Optional[int]) -> Cell: ...
    def with_formatter(self, formatter: List[settings.Formatter]) -> Cell: ...
    def append_formatter(self, formatter: List[settings.Formatter]): ...
    def with_appended_formatter(self, formatter: List[settings.Formatter]) -> Cell: ...


class Table:
    def __init__(self, data: List[List[Cell]]) -> Table:
        self.table: List[List[Cell]]
        self.border: settings.Border
    @staticmethod
    def create(header: List[Cell], content: List[List[Cell]], splitter: bool) -> Table: ...

    def with_border(self, border: settings.Border) -> Table: ...
    def render(self, setting: settings.Renderer) -> str: ...


from . import tabler
from typing import Iterable, Any


def from_plaintext(text: Iterable[Iterable[str]]) -> tabler.table.Table:
    """
    Create a table from some plain text.
    """
    tabler.table.Table(
        list(
            map(
                lambda row: list(
                    map(lambda s: tabler.table.Cell(tabler.table.Content(s)), row)
                ),
                text,
            )
        )
    )


def create_from_plaintext(
    header: Iterable[str], content: Iterable[Iterable[str]], splitter: bool
) -> tabler.table.Table:
    """
    Create a table from header and content.
    """
    tabler.table.Table.create(
        header=list(
            map(lambda cell: tabler.table.Cell(tabler.table.Content(cell)), header)
        ),
        content=list(
            map(
                lambda row: list(
                    map(lambda s: tabler.table.Cell(tabler.table.Content(s)), row)
                ),
                content,
            )
        ),
        splitter=splitter,
    )


def auto_cell(v: Any) -> tabler.table.Cell:
    """
    Automatically create a new table cell.
    """
    if v is None:
        return tabler.table.Cell(tabler.table.Content.Empty)
    elif isinstance(v, tabler.table.Cell):
        return v
    elif isinstance(v, tabler.table.Content):
        return tabler.table.Cell(v)
    elif isinstance(v, str):
        return tabler.table.Cell(tabler.table.Content(v))
    else:
        return tabler.table.Cell(tabler.table.Content(str(v)))


def auto_table_from(v: Iterable[Iterable[Any]]) -> tabler.table.Table:
    """
    Create a table with automatical cells.
    """
    return tabler.table.Table(
        list(map(lambda row: list(map(lambda cell: auto_cell(cell), row)), v))
    )


def auto_table_create(
    header: Iterable[Iterable[Any]], content: Iterable[Iterable[Any]], splitter: bool
) -> tabler.table.Table:
    """
    Create a table with automatical cells.
    """
    return tabler.table.Table.create(
        header=list(map(lambda cell: auto_cell(cell), header)),
        content=list(
            map(lambda row: list(map(lambda cell: auto_cell(cell), row)), content)
        ),
        splitter=splitter,
    )



class Align:
    Left: Align
    Right: Align
    Center: Align

class Overflow:
    Hide: Overflow
    Ellipse: Overflow

class Padding:
    Empty: Padding
    def __init__(self, left: int, right: int) -> Padding:
        self.left: int
        self.right: int

class Color:
    Black: Color
    Red: Color
    Green: Color
    Yellow: Color
    Blue: Color
    Magenta: Color
    Purple: Color
    Cyan: Color
    White: Color
    BrightBlack: Color
    BrightRed: Color
    BrightGreen: Color
    BrightYellow: Color
    BrightBlue: Color
    BrightMagenta: Color
    BrightPurple: Color
    BrightCyan: Color
    BrightWhite: Color
    @staticmethod
    def from_color_name(s: str) -> Color: ...

class Formatter:
    Clear: Formatter
    Normal: Formatter
    Bold: Formatter
    Dimmed: Formatter
    Italic: Formatter
    Underline: Formatter
    Blink: Formatter
    Reversed: Formatter
    Hidden: Formatter
    Strikethrough: Formatter
    @staticmethod
    def color(color: Color) -> Formatter: ...
    @staticmethod
    def rbg_color(r: int, g: int, b: int) -> Formatter: ...
    @staticmethod
    def on_color(color: Color) -> Formatter: ...
    @staticmethod
    def on_rbg_color(r: int, g: int, b: int) -> Formatter: ...

class Border:
    Default: Border
    Horizontal: Border
    Vertical: Border
    All: Border
    def __init__(self, left: bool, right: bool, top: bool, bottom: bool) -> Border:
        self.left: bool
        self.right: bool
        self.top: bool
        self.bottom: bool

class Renderer:
    Normal: Renderer
    Raw: Renderer
    Markdown: Renderer
